/*
** Module   : Dereferenceable.java
** Abstract : An interface for handles supporting deferenciation (:: operator).
**
** Copyright (c) 2013-2014, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- ---------------------------------Description---------------------------------
** 001 OM  20130220 Created initial version.
** 002 OM  20130220 Added support for dereference as left-side of an assignment.
*/

package com.goldencode.p2j.util;

/**
 * This interface declares only one method, dereference() which is the correspondent of the
 * Progress 10.x <code>::</code> punctuation. In the left-hand side it has a handle to a 
 * container object of some kind (buffer, table or ProDataSet) and the right-hand side is not 
 * an attribute or method, but instead is a named member of the left-hand side.
 */
public interface Dereferenceable
{
   /**
    * Dereference a named member from this collection. If this collection is a ProDataSet then 
    * the returned value is a {@link handle} to a named buffer, if the collection is a buffer
    * the returned value is the field value with the respective name. In the case that this 
    * collection does not have a named member with the name specified <code>memberName</code>
    * the unknown value is returned.
    *  
    * @param   memberName
    *          The name of the member to be extracted.
    *
    * @return  The value of the member or unknown (?) if there is no such member.
    */
   public BaseDataType dereference(String memberName);

   /**
    * This overloaded method is used when the dereference is the left-side of an assignment.
    * The implementation will extract the member from the collection and assign it the new value.
    *
    * @param   memberName
    *          The name of the member to be extracted.
    * @param   value
    *          The new value that will be assigned to the extracted field.
    */
   public void dereference(String memberName, Object value);
}
